@@ -49,7 +49,6 @@ import org.apache.cassandra.service.pager.PagingState;
 import org.apache.cassandra.transport.messages.ResultMessage;
 import org.apache.cassandra.utils.ByteBufferUtil;
 import org.apache.cassandra.utils.FBUtilities;
-import org.apache.cassandra.utils.Pair;
 import org.apache.cassandra.utils.UUIDGen;
 
 import static java.lang.String.format;
@@ -133,12 +132,13 @@ public abstract class DescribeStatement<T> extends CQLStatement.Raw implements C
     @Override
     public ResultMessage executeLocally(QueryState state, QueryOptions options)
     {
-        Pair<Keyspaces, UUID> schema = Schema.instance.getSchemaBlocking();
+        Keyspaces keyspaces = Schema.instance.distributedAndLocalKeyspaces();
+        UUID schemaVersion = Schema.instance.getVersion();
 
-        Keyspaces keyspaces = Keyspaces.builder()
-                                       .add(schema.left)
-                                       .add(VirtualKeyspaceRegistry.instance.virtualKeyspacesMetadata())
-                                       .build();
+        keyspaces = Keyspaces.builder()
+                             .add(keyspaces)
+                             .add(VirtualKeyspaceRegistry.instance.virtualKeyspacesMetadata())
+                             .build();
 
         PagingState pagingState = options.getPagingState();
 
@@ -156,7 +156,7 @@ public abstract class DescribeStatement<T> extends CQLStatement.Raw implements C
         //   (vint bytes) serialized schema hash (currently the result of Keyspaces.hashCode())
         //
 
-        long offset = getOffset(pagingState, schema.right);
+        long offset = getOffset(pagingState, schemaVersion);
         int pageSize = options.getPageSize();
 
         Stream<? extends T> stream = describe(state.getClientState(), keyspaces);
@@ -174,7 +174,7 @@ public abstract class DescribeStatement<T> extends CQLStatement.Raw implements C
 
         if (pageSize > 0 && rows.size() == pageSize)
         {
-            result.metadata.setHasMorePages(getPagingState(offset + pageSize, schema.right));
+            result.metadata.setHasMorePages(getPagingState(offset + pageSize, schemaVersion));
         }
 
         return new ResultMessage.Rows(result);
