@@ -21,7 +21,7 @@ import org.junit.After;
 import java.io.IOException;
 import java.nio.file.Files;
 import java.nio.file.Path;
-import java.util.Map;
+import java.util.Collections;
 
 import org.junit.Assert;
 import org.junit.Before;
@@ -717,6 +717,34 @@ public class AuditLoggerTest extends CQLTester
         assertEquals(0, AuthEvents.instance.listenerCount());
     }
 
+    @Test
+    public void testJMXArchiveCommand()
+    {
+        disableAuditLogOptions();
+        AuditLogOptions options = new AuditLogOptions();
+
+        try
+        {
+            StorageService.instance.enableAuditLog("BinAuditLogger", Collections.emptyMap(), "", "", "", "",
+                                                   "", "", 10, true, options.roll_cycle,
+                                                   1000L, 1000, "/xyz/not/null");
+            fail("not allowed");
+        }
+        catch (ConfigurationException e)
+        {
+            assertTrue(e.getMessage().contains("Can't enable audit log archiving via nodetool"));
+        }
+
+        options.archive_command = "/xyz/not/null";
+        options.audit_logs_dir = "/tmp/abc";
+        DatabaseDescriptor.setAuditLoggingOptions(options);
+        StorageService.instance.enableAuditLog("BinAuditLogger", Collections.emptyMap(), "", "", "", "",
+                                               "", "", 10, true, options.roll_cycle,
+                                               1000L, 1000, null);
+        assertTrue(AuditLogManager.instance.isEnabled());
+        assertEquals("/xyz/not/null", AuditLogManager.instance.getAuditLogOptions().archive_command);
+    }
+
     /**
      * Helper methods for Audit Log CQL Testing
      */
